@@ -9,6 +9,7 @@ validated and rendered.
 :license: BSD, see LICENSE for more details.
 """
 
+import itertools
 import urlparse
 
 from django.http import QueryDict
@@ -83,6 +84,21 @@ class Interface(object):
     def to_string(self, event):
         return ''
 
+    def get_search_context(self, event):
+        """
+        Returns a dictionary describing the data that should be indexed
+        by the search engine. Several fields are accepted:
+
+        - text: a list of text items to index as part of the generic query
+        - filters: a map of fields which are used for precise matching
+        """
+        return {
+            # 'text': ['...'],
+            # 'filters': {
+            #     'field": ['...'],
+            # },
+        }
+
 
 class Message(Interface):
     def __init__(self, message, params=()):
@@ -95,6 +111,11 @@ class Message(Interface):
             'params': self.params,
         }
 
+    def get_search_context(self, event):
+        return {
+            'text': [self.message] + list(self.params),
+        }
+
 
 class Query(Interface):
     def __init__(self, query, engine=None):
@@ -107,6 +128,11 @@ class Query(Interface):
             'engine': self.engine,
         }
 
+    def get_search_context(self, event):
+        return {
+            'text': [self.query],
+        }
+
 
 class Stacktrace(Interface):
     def __init__(self, frames):
@@ -170,6 +196,11 @@ class Stacktrace(Interface):
 
         return '\n'.join(result)
 
+    def get_search_context(self, event):
+        return {
+            'text': list(itertools.chain(*[[f['filename'], f['function'], f['context_line']] for f in self.frames])),
+        }
+
 
 class Exception(Interface):
     def __init__(self, value, type=None, module=None):
@@ -195,6 +226,11 @@ class Exception(Interface):
             'exception_module': self.module,
         })
 
+    def get_search_context(self, event):
+        return {
+            'text': [self.value, self.type, self.module]
+        }
+
 
 class Http(Interface):
     score = 100
@@ -285,6 +321,13 @@ class Http(Interface):
             'env': self.env,
         })
 
+    def get_search_context(self, event):
+        return {
+            'filters': {
+                'url': [self.url],
+            }
+        }
+
 
 class Template(Interface):
     def __init__(self, filename, context_line, lineno, pre_context=None, post_context=None,
@@ -328,6 +371,11 @@ class Template(Interface):
 
         return '\n'.join(result)
 
+    def get_search_context(self, event):
+        return {
+            'text': [self.abs_path, self.filename, self.context_line, self.pre_context, self.post_context],
+        }
+
 
 class User(Interface):
     pass
